/**************************************************************************************************

accessMinisatP.c - modified Main.C from (C) Niklas Een, Niklas Sörensson, 2004

**************************************************************************************************/


#include <cstdarg>
#include <unistd.h>
#include <signal.h>
#include "MiniSat.h"
#include "PbSolver.h"
#include "Debug.h"

#include "accessGpw.h"

// convenient for some output
#include<iostream>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

using namespace std;

//=================================================================================================
// Options referenced by other files ...

bool     opt_satlive   = true;
bool     opt_ansi      = true;
char*    opt_cnf       = NULL;
int      opt_verbosity = 1;
bool     opt_try       = false;     // (hidden option -- if set, then "try" to parse, but don't output "s UNKNOWN" if you fail

SolverT  opt_solver        = st_MiniSat;
ConvertT opt_convert       = ct_Mixed;
ConvertT opt_convert_goal  = ct_Undef;
bool     opt_convert_weak  = true;
bool     opt_avoid_band_constraint = false;
//bool     opt_convert_gpw   = true;
//bool     opt_convert_2gpw  = false;
GpwT     opt_convert_gpw   = gt_positive;
bool     opt_convert_bdd_monotonic = false;
//bool     opt_convert_bdd_binary_decomposition = false;
int      opt_convert_bdd_decomposition = 0;  // 1:binary base, 2:multi base
bool     opt_convert_bdd_interval = false;
bool     opt_convert_bdd_increasing_order = false;
double   opt_bdd_thres     = 3;
double   opt_sort_thres    = 20;
double   opt_goal_bias     = 3;
Int      opt_goal          = Int_MAX;
Command  opt_command       = cmd_Minimize;
bool     opt_binary_minimization = true;
bool     opt_band_for_goal = false;
bool     opt_goal_gcd = true;
bool     opt_branch_pbvars = false;
int      opt_polarity_sug  = 1;
bool     opt_old_format    = false;
bool     opt_extract_ucore = false;
int      opt_ucore_mode    = 3;

bool     opt_model_check   = false;

char*    opt_input  = NULL;
char*    opt_result = NULL; 

int      stats_bdd_cost = 0;
int      stats_sort_cost = 0;
int      stats_adder_cost = 0;
int      stats_monotonic_coding = 0, stats_non_monotonic_coding = 0;
int      stats_std_form = 0, stats_band_form = 0;
int      stats_sat_calls = 0, stats_unsat_calls = 0;
int      stats_bdd_nodes = 0;
double   stats_cnf_coding_time;

//=================================================================================================

// these are signal handlers form main.c of minisat+ - no idea if they are used, lets see
static void SIGINT_handler(int signum) {
    reportf("\n");
    reportf("*** INTERRUPTED ***\n");
    SatELite::deleteTmpFiles();
    //_exit(0); 
}     // (using 'exit()' rather than '_exit()' sometimes causes the solver to hang (why?))


static void SIGTERM_handler(int signum) {
    reportf("\n");
    reportf("*** TERMINATED ***\n");
    //outputResult(*pb_solver, false);
    SatELite::deleteTmpFiles();
    //_exit(pb_solver->best_goalvalue == Int_MAX ? 0 : 10); 
}


PbSolver*   pb_solver = NULL;
vec<Lit> out_ps; vec<Int> out_cs; 

//possibly get more options passed
//annoying: user has to pass number of constraints and variables
void initPbSolving(int n_var, int n_con, int cmd=0){
    //cout << " in init solving with " << n_var << " vars " << endl;
    pb_solver = NULL;
    pb_solver = new PbSolver();

    signal(SIGINT , SIGINT_handler);
    signal(SIGTERM, SIGTERM_handler);

    pb_solver->allocConstrs(n_var, n_con);
    
    // Set command from 'PBSATISFIABILITYONLY':
    char* value = getenv("PBSATISFIABILITYONLY");
    if (value != NULL && atoi(value) == 1)
        reportf("Setting switch '-first' from environment variable 'PBSATISFIABILITYONLY'.\n"),
        opt_command = cmd_FirstSolution;
}

bool oneof(cchar* arg, cchar* alternatives)
{
    // Force one leading '-', allow for two:
    if (*arg != '-') return false;
    arg++;
    if (*arg == '-') arg++;

    // Scan alternatives:
    vec<char*>  alts;
    splitString(alternatives, ",", alts);
    for (int i = 0; i < alts.size(); i++){
        if (strcmp(arg, alts[i]) == 0){
            xfreeAll(alts);
            return true;
        }
    }
    xfreeAll(alts);
    return false;
}

void setOption(char o[]){ 
  // vec<char*>  args;   // Non-options

  //    for (int i = 1; i < argc; i++){
        char*   arg = o; //argv[i];
        if (arg[0] == '-'){
            if (oneof(arg,"h,help")) fprintf(stderr, "invalid option"), exit(0);

            else if (oneof(arg, "M,minisat"     )) opt_solver = st_MiniSat;
            else if (oneof(arg, "S,satelite"    )) opt_solver = st_SatELite;
            else if (oneof(arg, "GM,glueminisat")) opt_solver = st_GlueMiniSat;
            else if (oneof(arg, "M2,minisat2.2" )) opt_solver = st_MiniSat22;
            else if (oneof(arg, "GL,glucose1.0" )) opt_solver = st_Glucose10;

            else if (oneof(arg, "ca,adders" )) opt_convert = ct_Adders;
            else if (oneof(arg, "cs,sorters")) opt_convert = ct_Sorters;
            else if (oneof(arg, "cb,bdds"   )) opt_convert = ct_BDDs;
            else if (oneof(arg, "cm,mixed"  )) opt_convert = ct_Mixed;

            else if (oneof(arg, "ga,goal-adders" )) opt_convert_goal = ct_Adders;
            else if (oneof(arg, "gs,goal-sorters")) opt_convert_goal = ct_Sorters;
            else if (oneof(arg, "gb,goal-bdds"   )) opt_convert_goal = ct_BDDs;
            else if (oneof(arg, "gm,goal-mixed"  )) opt_convert_goal = ct_Mixed;

            else if (oneof(arg, "w,weak-off"     )) opt_convert_weak = false;
            else if (oneof(arg, "m,band-off" )) opt_avoid_band_constraint = true;

            else if (oneof(arg, "wg,without-gpw" )) opt_convert_gpw = gt_none;
                                                       // normal sorter
            else if (oneof(arg, "gpw-pos"   )) opt_convert_gpw = gt_positive;
            else if (oneof(arg, "gpw-neg"   )) opt_convert_gpw = gt_negative;
            else if (oneof(arg, "gpw-lo"    )) opt_convert_gpw = gt_low;
            else if (oneof(arg, "gpw-hi"    )) opt_convert_gpw = gt_high;
            else if (oneof(arg, "gpw-both"  )) opt_convert_gpw = gt_both;
            else if (oneof(arg, "bdd-m"     )) opt_convert_bdd_monotonic = true;
        //            else if (oneof(arg, "bdd-b"     )) opt_convert_bdd_binary_decomposition = true;
        else if (oneof(arg, "bdd-b"     )
                 && opt_convert_bdd_decomposition==0)
                                           opt_convert_bdd_decomposition = 1;
        else if (oneof(arg, "bdd-d"    )
                     && opt_convert_bdd_decomposition==0)
                                               opt_convert_bdd_decomposition = 2;
            else if (oneof(arg, "bdd-i"     )) opt_convert_bdd_interval = true;
            else if (oneof(arg, "bdd-r"     )) opt_convert_bdd_increasing_order = true;
            else if (oneof(arg, "mc,model-check" )) opt_model_check = true;

            //(make nicer later)
            else if (strncmp(arg, "-bdd-thres=" , 11) == 0) opt_bdd_thres  = atof(arg+11);
            else if (strncmp(arg, "-sort-thres=", 12) == 0) opt_sort_thres = atof(arg+12);
            else if (strncmp(arg, "-goal-bias=",  11) == 0) opt_goal_bias  = atof(arg+11);
            else if (strncmp(arg, "-goal="     ,   6) == 0) opt_goal       = atoi(arg+ 6);  // <<== real bignum parsing here
            else if (strncmp(arg, "-cnf="      ,   5) == 0) opt_cnf        = arg + 5;
            //(end)

            else if (oneof(arg, "1,first"   )) opt_command = cmd_FirstSolution;
            else if (oneof(arg, "A,all"     )) opt_command = cmd_AllSolutions;
            else if (oneof(arg, "seq"       )) opt_binary_minimization = false;
            else if (oneof(arg, "gg,goal-gcd-off")) opt_goal_gcd = false;
            else if (oneof(arg, "bg,goal-band")) opt_band_for_goal = true;

            else if (oneof(arg, "p,pbvars"  )) opt_branch_pbvars = true;
            else if (oneof(arg, "ps+"       )) opt_polarity_sug = +1;
            else if (oneof(arg, "ps-"       )) opt_polarity_sug = -1;
            else if (oneof(arg, "ps0"       )) opt_polarity_sug =  0;

            else if (oneof(arg, "of,old-fmt" )) opt_old_format = true;

            else if (oneof(arg, "u,ucore="   )) opt_extract_ucore = true;
            else if (strncmp(arg, "-ucore-mode=",  12) == 0) opt_ucore_mode  = atof(arg+12);
            else if (oneof(arg, "s,satlive" )) opt_satlive = false;
            else if (oneof(arg, "a,ansi"    )) opt_ansi    = false;
            else if (oneof(arg, "try"       )) opt_try     = true;
            else if (oneof(arg, "v0"        )) opt_verbosity = 0;
            else if (oneof(arg, "v1"        )) opt_verbosity = 1;
            else if (oneof(arg, "v2"        )) opt_verbosity = 2;

            else
                fprintf(stderr, "ERROR! Invalid command line option: %s\n", arg), exit(1);

        } // else args.push(arg);
    //}

    if (!opt_avoid_band_constraint && opt_convert_bdd_monotonic && !opt_convert_bdd_interval)
      reportf("We set -m because of -bdd-m.\n"), opt_avoid_band_constraint = true;

}

void addConstraint(char *ps[], int *cs, int n_p, int rhs, int ineq){
    Lit dlt=lit_Undef;
    out_ps.clear(); // global var that is cleared instead of local one - 
    out_cs.clear(); // avoid repeated memory allocation
    //out_ps.push(Lit(solver.getVar(parseIdent(in, tmp)))); 
    //actually, they use solver.getVar(...) here ... should we rely on OCaml user?
    int i;
    //char string[50];
    for (i=0; i<n_p; i++){
        //cout << ps[i] <<endl;
        dlt = Lit(pb_solver->getVar(ps[i]));
        out_ps.push(dlt);
        out_cs.push(cs[i]);
    }
    if (ineq == 3) {
     pb_solver->addGoal(out_ps, out_cs);
    } else {
    //pb_solver->addConstr(out_ps, out_cs, rhs, ineq, Int_MAX, -2, dlt, dlt, true);
    pb_solver->addConstr(out_ps, out_cs, rhs, ineq);
    }
//    cout <<"Adding constraint w "<<n_p<<" components, rhs "<<rhs<<" of type "<<ineq<<endl;
}

PbSolver::solve_Command convert(Command cmd) {
    switch (cmd){
    case cmd_Minimize:      return PbSolver::sc_Minimize;
    case cmd_FirstSolution: return PbSolver::sc_FirstSolution;
    case cmd_AllSolutions:  return PbSolver::sc_AllSolutions;
    default: assert(false); return PbSolver::sc_Minimize; //Dummy
    }
}

int solvePb(char *result[]){
    int true_vars=0;
    bool optimum=true;

    // some checks to ensure option consistency
    /*if (!opt_keep_monotonic && opt_convert_bdd_monotonic)
      reportf("We set -m because of -bdd-m.\n"), opt_keep_monotonic = true;

    if (!opt_keep_monotonic && opt_convert_bdd_interval)
      reportf("We set -m because of -bdd-i.\n"), opt_keep_monotonic = true;*/

    if (opt_extract_ucore
	&& opt_solver != st_GlueMiniSat
	&& opt_solver != st_MiniSat22
	&& opt_solver != st_Glucose10)
      reportf("We set -GM because of -extract-ucore.\n"), opt_solver = st_GlueMiniSat;


    // actual solving
    pb_solver->solve(*pb_solver, convert(opt_command));

    if (pb_solver->goal == NULL && pb_solver->best_goalvalue != Int_MAX)
        opt_command = cmd_FirstSolution;    // (otherwise output will be wrong)
    if (!pb_solver->okay())
        opt_command = cmd_Minimize;         // (HACK: Get "UNSATISFIABLE" as output)


    if (opt_command == cmd_FirstSolution)
        optimum=false;



    if (optimum){
        if (pb_solver->best_goalvalue == Int_MAX) 
	    { //printf("s UNSATISFIABLE\n");
	    true_vars=-1;
	    }
        else                             {/*printf("s OPTIMUM FOUND\n");*/}
    }else{
        if (pb_solver->best_goalvalue == Int_MAX) 
	    {
	    //printf("s UNKNOWN\n");
	    true_vars=-2;
	    }
        else                             {/*printf("s SATISFIABLE\n");*/ }
    }

    if (pb_solver->best_goalvalue != Int_MAX){
        //printf(" result found: ");
        for (int i = 0; i < pb_solver->best_model.size(); i++){
  //          printf(" %s%s", pb_solver->best_model[i]?"":"-", pb_solver->index2name[i]);
	    if (pb_solver->best_model[i]){
		//result[true_vars] = malloc(30);
	        result[true_vars++] = (char*) (pb_solver->index2name[i]);
		}
	    }
    //    printf("\n");
    }
    //delete (pb_solver);
    signal(SIGINT , SIG_DFL);
    signal(SIGTERM, SIG_DFL);
    return true_vars;
}



void reportf(const char* format, ...)
{
    static bool col0 = true;
    static bool bold = false;
    va_list args;
    va_start(args, format);
    char* text = vnsprintf(format, args);
    va_end(args);

    for(char* p = text; *p != 0; p++){
        if (col0 && opt_satlive)
            putchar('c'), putchar(' ');

        if (*p == '\b'){
            bold = !bold;
            col0 = false;
        }else{
            putchar(*p);
            col0 = (*p == '\n' || *p == '\r');
        }
    }
    fflush(stdout);
}

void printStats(BasicSolverStats& stats, double cpu_time)
{
    reportf("std/band forms        : %d,%d\n", stats_std_form, stats_band_form);
    reportf("2cl/3cl ITE-codings   : %d,%d\n", stats_monotonic_coding, stats_non_monotonic_coding);
    reportf("BDD/srt/adr     costs : %d,%d,%d\n", stats_bdd_cost,stats_sort_cost,stats_adder_cost);
    reportf("BDD nodes             : %d\n", stats_bdd_nodes);
    reportf("SAT/UNSAT calls       : %d,%d\n", stats_sat_calls,stats_unsat_calls);
    reportf("restarts              : %"I64_fmt"\n", stats.starts);
    reportf("conflicts             : %-12"I64_fmt"   (%.0f /sec)\n", stats.conflicts   , stats.conflicts   /cpu_time);
    reportf("decisions             : %-12"I64_fmt"   (%.0f /sec)\n", stats.decisions   , stats.decisions   /cpu_time);
    reportf("propagations          : %-12"I64_fmt"   (%.0f /sec)\n", stats.propagations, stats.propagations/cpu_time);
    reportf("inspects              : %-12"I64_fmt"   (%.0f /sec)\n", stats.inspects    , stats.inspects    /cpu_time);
    reportf("CPU time (CNF coding) : %g s            (%g s)\n", cpu_time, stats_cnf_coding_time);
}
